﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

//Copyright[2018][Andrew Polar]

//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at

//    http://www.apache.org/licenses/LICENSE-2.0

//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.

//data source http://homes.esat.kuleuven.be/~smc/daisy/daisydata.html

namespace UT_Cons
{
    class Program
    {
        static int N_code = 1;

        static void ShowSignal(string name, int columns, double[] p)
        {
            Console.WriteLine();
            Console.WriteLine(name);
            int cnt = 0;
            foreach (double x in p)
            {
                if (x >= 0)
                {
                    Console.Write("{0: 0.000}   ", x);
                }
                else
                {
                    Console.Write("{0:0.000}   ", x);
                }
                if (++cnt >= columns)
                {
                    Console.WriteLine();
                    cnt = 0;
                }
            }
            Console.WriteLine();
        }

        private static bool ShowInfoData(string fileName)
        {
            if (!File.Exists(fileName))
            {
                Console.WriteLine("File {0} not found", fileName);
                return false;
            }
            StreamReader file = new StreamReader(fileName);
            string line;
            while ((line = file.ReadLine()) != null)
            {
                System.Console.WriteLine(line);
            }
            file.Close();
            return true;
        }

        static void DoRobotArm()
        {
            string fileName = @"..\..\..\..\robot_arm.dat\robot_arm.dat";
            string infoFile = @"..\..\..\..\robot_arm.dat\readme.txt";
            if (!ShowInfoData(infoFile))
            {
                Console.WriteLine("Warning info file is missing");
            }
            DataReader dr = new DataReader(fileName);
            if (!dr.ReadData(2))
            {
                Environment.Exit(N_code++);
            }
            if (dr.GetSize() < 100)
            {
                Console.WriteLine("Data size is too short {0}", dr.GetSize());
                Environment.Exit(N_code++);
            }
            double[] x = dr.GetColumn(0);
            if (null == x)
            {
                Environment.Exit(N_code++);
            }
            double[] y = dr.GetColumn(1);
            if (null == y)
            {
                Environment.Exit(N_code++);
            }
            Console.WriteLine("Data is read");
            ShowSignal("Input", 10, x);
            ShowSignal("Output", 10, y);

            U1 u1 = new U1();
            u1.SetSizes(40, 50);
            if (!u1.NormalizeInput(x))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.Indentification(y, 20))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.CompareOutputs(y))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.ShowOperator())
            {
                Environment.Exit(N_code++);
            }
        }

        static void DoFlutter()
        {
            string fileName = @"..\..\..\..\flutter.dat\flutter.dat";
            string infoFile = @"..\..\..\..\flutter.dat\readme.txt";
            if (!ShowInfoData(infoFile))
            {
                Console.WriteLine("Warning info file is missing");
            }
            DataReader dr = new DataReader(fileName);
            if (!dr.ReadData(2))
            {
                Environment.Exit(N_code++);
            }
            if (dr.GetSize() < 100)
            {
                Console.WriteLine("Data size is too short {0}", dr.GetSize());
                Environment.Exit(N_code++);
            }
            double[] x = dr.GetColumn(0);
            if (null == x)
            {
                Environment.Exit(N_code++);
            }
            double[] y = dr.GetColumn(1);
            if (null == y)
            {
                Environment.Exit(N_code++);
            }
            Console.WriteLine("Data is read");
            ShowSignal("Input", 10, x);
            ShowSignal("Output", 10, y);

            U1 u1 = new U1();
            u1.SetSizes(45, 40);
            if (!u1.NormalizeInput(x))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.Indentification(y, 5))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.CompareOutputs(y))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.ShowOperator())
            {
                Environment.Exit(N_code++);
            }
        }

        static void DoDryer()
        {
            string fileName = @"..\..\..\..\dryer.dat\dryer.dat";
            string infoFile = @"..\..\..\..\dryer.dat\readme.txt";
            if (!ShowInfoData(infoFile))
            {
                Console.WriteLine("Warning info file is missing");
            }
            DataReader dr = new DataReader(fileName);
            if (!dr.ReadData(2))
            {
                Environment.Exit(N_code++);
            }
            if (dr.GetSize() < 100)
            {
                Console.WriteLine("Data size is too short {0}", dr.GetSize());
                Environment.Exit(N_code++);
            }
            double[] x = dr.GetColumn(0);
            if (null == x)
            {
                Environment.Exit(N_code++);
            }
            double[] y = dr.GetColumn(1);
            if (null == y)
            {
                Environment.Exit(N_code++);
            }
            Console.WriteLine("Data is read");
            ShowSignal("Input", 10, x);
            ShowSignal("Output", 10, y);

            U1 u1 = new U1();
            u1.SetSizes(20, 20);
            if (!u1.NormalizeInput(x))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.Indentification(y, 1))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.CompareOutputs(y))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.ShowOperator())
            {
                Environment.Exit(N_code++);
            }
        }

        static void DoBallBeam()
        {
            string fileName = @"..\..\..\..\ballbeam.dat\ballbeam.dat";
            string infoFile = @"..\..\..\..\ballbeam.dat\readme.txt";
            if (!ShowInfoData(infoFile))
            {
                Console.WriteLine("Warning info file is missing");
            }
            DataReader dr = new DataReader(fileName);
            if (!dr.ReadData(2))
            {
                Environment.Exit(N_code++);
            }
            if (dr.GetSize() < 100)
            {
                Console.WriteLine("Data size is too short {0}", dr.GetSize());
                Environment.Exit(N_code++);
            }
            double[] x = dr.GetColumn(0);
            if (null == x)
            {
                Environment.Exit(N_code++);
            }
            double[] y = dr.GetColumn(1);
            if (null == y)
            {
                Environment.Exit(N_code++);
            }
            Console.WriteLine("Data is read");
            ShowSignal("Input", 10, x);
            ShowSignal("Output", 10, y);

            U1 u1 = new U1();
            u1.SetSizes(70, 100);
            if (!u1.NormalizeInput(x))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.Indentification(y, 5))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.CompareOutputs(y))
            {
                Environment.Exit(N_code++);
            }
            if (!u1.ShowOperator())
            {
                Environment.Exit(N_code++);
            }
        }

        static void Main(string[] args)
        {
            //DoRobotArm();
            DoFlutter();
            //DoDryer();
            //DoBallBeam();
        }
    }
}
