﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace UT_Cons
{
    class U1
    {
        private int? _tsize = null;
        private int? _xsize = null;
        private int[] _x = null;
        private double[,] U = null;

        public void SetSizes(int nt, int nx)
        {
            _tsize = nt;
            _xsize = nx;
        }

        public bool NormalizeInput(double[] x)
        {
            if (null == x)
            {
                Console.WriteLine("Invalid input");
                return false;
            }
            if (x.Length < 100)
            {
                Console.WriteLine("Input signal is too short");
                return false;
            }
            double xmin = x[0];
            double xmax = x[0];
            foreach (double p in x)
            {
                if (p > xmax) xmax = p;
                if (p < xmin) xmin = p;
            }
            Console.WriteLine("min/max for input {0} {1}", xmin, xmax);
            double? delta = (xmax - xmin) / (double)(_xsize - 1);
            _x = new int[x.Length];
            int cnt = 0;
            foreach (double p in x)
            {
                _x[cnt++] = (int)((p - xmin) / delta);
            }
            int min = _x[0];
            int max = _x[0];
            foreach (int p in _x)
            {
                if (p > max) max = p;
                if (p < min) min = p;
            }
            Console.WriteLine("min/max for lookup {0} {1}", min, max);
            return true;
        }

        public bool Indentification(double[] y, int startsize)
        {
            if (y.Length != _x.Length)
            {
                Console.WriteLine("input/output sizes mismatch");
                return false;
            }
            U = new double[(int)_xsize, (int)_tsize];
            for (int i = 0; i < _xsize; ++i)
            {
                for (int j = 0; j < _tsize; ++j)
                {
                    U[i, j] = 0.0;
                }
            }

            int counter = 0;
            int time = startsize;
            double oldTotalDelta = 1e99;
            while (true)
            {
                double totalDelta = 0.0;
                int cnt = 0;
                for (int i = (int)(time) - 1; i < _x.Length; ++i)
                {
                    double output = 0.0;
                    for (int j = 0; j < time; ++j)
                    {
                        output += U[_x[i - j], j];
                    }
                    double delta = (y[i] - output) / time;
                    totalDelta += delta * delta;
                    ++cnt;
                    for (int j = 0; j < time; ++j)
                    {
                        U[_x[i - j], j] += delta;
                    }
                }
                totalDelta /= (double)(cnt);
                totalDelta = Math.Sqrt(totalDelta);
                Console.WriteLine("Standard error in identification step {0}", totalDelta);

                if (++time > _tsize) time = (int)(_tsize);
                if (totalDelta / oldTotalDelta > 0.98 && time == (int)_tsize) break;
                else oldTotalDelta = totalDelta;
                if (++counter > 300) break;
            }

            return true;
        }

        public bool CompareOutputs(double[] y)
        {
            if (null == U)
            {
                Console.WriteLine("Operator is not identified");
                return false;
            }

            Console.WriteLine("\nActual and modelled outputs, first column is index\n");
            double diff = 0.0;
            double signal = 0.0;
            for (int i = (int)(_tsize) - 1; i < _x.Length; ++i)
            {
                double output = 0.0;
                for (int j = 0; j < _tsize; ++j)
                {
                    output += U[_x[i - j], j];
                }
                diff += (y[i] - output) * (y[i] - output);
                signal += y[i] * y[i];
                Console.WriteLine("{0:0000}  {1:0.000}  {2:0.000}", i, y[i], output);
            }
            double relativeError = Math.Sqrt(diff / signal);
            Console.WriteLine("Relative error {0}", relativeError);
            return true;
        }

        public bool ShowOperator()
        {
            if (null == U)
            {
                Console.WriteLine("Operator is not identified");
                return false;
            }
            Console.WriteLine("\nOperator\n");
            for (int i = 0; i < _tsize; ++i)
            {
                Console.WriteLine("time index {0}", i);
                for (int j = 0; j < _xsize; ++j)
                {
                    Console.Write("{0:0.0000} ", U[j, i]);
                }
                Console.WriteLine("\n");
            }
            return true;
        }
    }
}
